@@ -1,4 +1,5 @@
 /* hardlink.c - handles hardlinked files
+   This has nothing to do with cryptography.
    Copyright (C) 1998 Paul Sheer
 
    This program is free software; you can redistribute it and/or modify
@@ -20,6 +21,7 @@
 #include "mostincludes.h"
 #include "mirrordir.h"
 #include "vfs/vfs.h"
+#include "diffie/compat.h"
 #include "mad.h"
 
 
@@ -72,9 +74,11 @@ void add_hardlink (char *path, struct stat *s, struct stat *m)
     l = malloc (sizeof (struct hardlink_list));
 
     memset (l, 0, sizeof (struct hardlink_list));
-    l->name = strdup (path);
-    l->mirror_st_dev = m->st_dev;
-    l->mirror_st_ino = m->st_ino;
+    l->name = (char *) strdup (path);
+    if (m) {
+	l->mirror_st_dev = m->st_dev;
+	l->mirror_st_ino = m->st_ino;
+    }
     l->st_dev = s->st_dev;
     l->st_ino = s->st_ino;
     l->st_nlink = s->st_nlink;
@@ -86,8 +90,8 @@ void add_hardlink (char *path, struct stat *s, struct stat *m)
 
 int correct_link (struct stat *s)
 {
-    return s->st_dev == hl->mirror_st_dev &&
-	s->st_ino == hl->mirror_st_ino;
+    return memcmp (&s->st_dev, &hl->mirror_st_dev, sizeof (hl->mirror_st_dev)) == 0
+	&& s->st_ino == hl->mirror_st_ino;
 }
 
 int is_hardlink (struct stat *s)
@@ -107,12 +111,12 @@ int find_hardlink (struct stat *s)
 
 /* sometimes we search for things twice in a row,
    so lets check if its not already found */
-    if (hl->st_dev == s->st_dev
+    if (memcmp (&hl->st_dev, &s->st_dev, sizeof (hl->mirror_st_dev)) == 0
 	&& hl->st_ino == s->st_ino)
 	return 1;
     rewind (hl);
     for (;;) {
-	if (hl->st_dev == s->st_dev
+	if (memcmp (&hl->st_dev, &s->st_dev, sizeof (hl->st_dev)) == 0
 	    && hl->st_ino == s->st_ino)
 	    return 1;
 	if (!hl->next)
@@ -177,6 +181,11 @@ void create_link (char *q)
 	    progmess_strerror ("error trying to create hardlinks", q);
 }
 
+char *retrieve_link (void)
+{
+    return hl->name;
+}
+
 
 
 
