@@ -112,6 +112,10 @@ int gzip_backups = 0;
 int case_insensitive_filenames = 0;
 int files_to_lower = 0;
 int files_to_upper = 0;
+int test_login = 0;
+int read_password_from_stdin = 0;
+int no_warn_first_login = 0;
+extern int ftpfs_literal_password;
 
 char *ftpfs_password = 0;
 extern char *ftpfs_proxy_host;
@@ -883,11 +887,14 @@ int copy_regular_file (char *p, char *q, struct stat *s, struct stat *t)
 	    off_t count, c;
 	    char *b;
 	    be_nice_to_cpu ();
-	    mad_check (__FILE__, __LINE__);
-	    count = gzip < 0 ? gzread ((gzFile) f, buf, COPY_BUF_SIZE) : mc_read ((int) f, buf, COPY_BUF_SIZE);
-	    mad_check (__FILE__, __LINE__);
-	    if (count < 0) {
-		progmess_strerror ("error trying to read from file", p);
+	    do {
+		count = gzip < 0 ? gzread ((gzFile) f, buf, COPY_BUF_SIZE) : mc_read ((int) f, buf, COPY_BUF_SIZE);
+	    } while (count < 0 && errno == EINTR);
+	    if (count <= 0) {
+		if (!count)
+		    progmess ("error trying to read from file: premature end of file", p);
+		else
+		    progmess_strerror ("error trying to read from file", p);
 		f = gzip < 0 ? gzclose ((gzFile) f) : mc_close ((int) f);
 		g = gzip > 0 ? gzclose ((gzFile) g) : mc_close ((int) g);
 		return 1;
@@ -896,9 +903,14 @@ int copy_regular_file (char *p, char *q, struct stat *s, struct stat *t)
 	    togo -= count;
 	    offset += count;
 	    while (count > 0) {
-		c = gzip > 0 ? gzwrite ((gzFile) g, b, count) : mc_write ((int) g, b, count);
-		if (c < 0) {
-		    progmess_strerror ("error trying to write to file", q);
+		do {
+		    c = gzip > 0 ? gzwrite ((gzFile) g, b, count) : mc_write ((int) g, b, count);
+		} while (c < 0 && errno == EINTR);
+		if (c <= 0) {
+		    if (!count)
+			progmess ("error trying to write to file: no bytes written", q);
+		    else
+			progmess_strerror ("error trying to write to file", q);
 		    f = gzip < 0 ? gzclose ((gzFile) f) : mc_close ((int) f);
 		    g = gzip > 0 ? gzclose ((gzFile) g) : mc_close ((int) g);
 		    return 1;
@@ -1636,7 +1648,8 @@ char *p[256] =
 "    copydir -[abBdeFGhklMmNOopRrsTtvVXz] <path> [<path> ...] <dest>",
 "    recursdir -[abBdeFGhklMmNOopRrsTtvVXz] <path> [<path> ...]",
 "                                          [--tar-file <file>]",
-"    pslogin [-K [512|768|1024|1536]] mc:" PATH_SEP_STR PATH_SEP_STR "[user@]host[:port][/path]",
+"    pslogin [-K [512|768|1024|1536]] [--test-login]",
+"                                    mc:" PATH_SEP_STR PATH_SEP_STR "[user@]host[:port][/path]",
 "",
 "But usually just",
 "    mirrordir [--exclude <path>] <control> <mirror>",
@@ -1721,7 +1734,11 @@ char *p[256] =
 "",
 "  --no-use-passive-connections  don't use passive ftp connections",
 "",
-"  -p, --password <password>     ftp password",
+"  -p, --password <password>     ftp, pslogin or mc:// password",
+"  -P, --password-exact <passwd> never prepend a `-' to anonymous password",
+"      --test-login              do not login, just return exit status",
+"      --no-warn-first-login     don't warn on first pslogin ever to a machine",
+"      --read-password-from-stdin  used for non-interactive invocation",
 "      --netrc                   consult ~/.netrc (this is the default)",
 "      --no-netrc                don't consult ~/.netrc",
 "",
@@ -2011,7 +2028,9 @@ static char *load_file (char *f)
     while (s.st_size > 0) {
 	int count;
 	count = read (fd, r, s.st_size);
-	if (count < 0)
+	if (count < 0 && errno == EINTR)
+	    continue;
+	if (count <= 0)
 	    break;
 	s.st_size -= count;
 	r += count;
@@ -2214,6 +2233,11 @@ int pars_opts (int argc, char **argv)
 	{0, "", "--to-lower", ARG_SET, 0, 0, &files_to_lower},
 	{0, "", "--to-upper", ARG_SET, 0, 0, &files_to_upper},
 	{0, "", "--no-use-passive-connections", ARG_CLEAR, 0, 0, &ftpfs_use_passive_connections},
+	{0, "", "--test-login", ARG_SET, 0, 0, &test_login},
+	{0, "", "--no-warn-first-login", ARG_SET, 0, 0, &no_warn_first_login},
+	{0, "", "--read-password-from-stdin", ARG_SET, 0, 0, &read_password_from_stdin},
+	{0, "", "--mirror-mode", ARG_IGNORE, 0, 0, 0},
+	{'P', "", "--password-exact", ARG_SET, 0, 0, &ftpfs_literal_password},
 	{0, 0, 0, 0, 0, 0, 0}
     };
 
@@ -2230,7 +2254,7 @@ int pars_opts (int argc, char **argv)
     getwd (cwd);
 #endif
 
-    if (strcmp (progname, "mirrordir") && strncmp (progname, "MIRR", 4)) {
+    if (strcmp (progname, "mirrordir") && strcmp (progname, "lt-mirrordir") && strncmp (progname, "MIRR", 4) && strcmp (argv[1] ? argv[1] : "", "--mirror-mode")) {
 /* defaults for copydir */
 	dont_remove = 1;
 	copy_mode = 1;
@@ -2429,6 +2453,15 @@ char *password_callback (char *a, char *b, char *c, char *host, char *user)
 	    verbmess ("found password in ~/.netrc for", host);
 	return p;
     }
+    if (read_password_from_stdin) {
+	char s[20], *q;
+	memset (s, 0, 20);
+	read (0, s, 16);
+	q = strchr (s, '\n');
+	if (q)
+	    *q = '\0';
+	return (char *) strdup (s);
+    }
     /* if everything else fails, then ask */
     return (char *) strdup ((char *) getpass (b));
 }
@@ -2484,6 +2517,8 @@ static RETSIGTYPE term_handler (int x)
 static int warn_callback (char *f, void *x)
 {
     char c[20] = "";
+    if (no_warn_first_login)
+	return 0;
     printf (\
     "You are attempting to connect to a host that you appear to have\n" \
      "never connected to before as this user on this machine. If you\n" \
@@ -2500,7 +2535,7 @@ static int warn_callback (char *f, void *x)
     return 1; /* prevents warning */
 }
 
-int mcfs_open_login (char *path);
+int mcfs_open_login (char *path, int test_login);
 void add_functions (void);
 
 int *debug = 0;
@@ -2509,15 +2544,16 @@ int *debug = 0;
 
 void do_download_scripts (char *argv_0)
 {
-    struct stat s;
+    struct stat st;
     char c[20] = "";
-    if (stat (SCRIPT_ACCEPT, &s)) {
+    memset (&st, 0, sizeof (struct stat));
+    if (stat (SCRIPT_ACCEPT, &st)) {
 	progmess ("Cannot locate script " SCRIPT_ACCEPT, "Mirrordir has not been installed properly");
 	exit (1);
     }
     if (download_scripts)
 	goto do_download;
-    if (s.st_size < SCRIPT_ACCEPT_MIN_SIZE) {
+    if (st.st_size < SCRIPT_ACCEPT_MIN_SIZE) {
 	int f;
 	printf ("Mirrordir has detected that " SCRIPT_ACCEPT " is truncated.\n");
 	printf ("This means that you have a US version of this software.\n");
@@ -2559,8 +2595,8 @@ void do_download_scripts (char *argv_0)
 	    write (f, "                                                                                 ", 80);
 	    close (f);
 	    system (t);
-	    stat (SCRIPT_CONNECT, &s);
-	    if (s.st_size < SCRIPT_ACCEPT_MIN_SIZE || download_scripts) {
+	    stat (SCRIPT_CONNECT, &st);
+	    if (st.st_size < SCRIPT_ACCEPT_MIN_SIZE || download_scripts) {
 		printf ("One of your scripts is still truncated - download must have failed.\n");
 		printf ("This message will not display in the future. Use\n");
 		printf ("\tmirrordir --download-scripts\nto force a download.\n");
@@ -2577,6 +2613,8 @@ void do_download_scripts (char *argv_0)
     }
 }
 
+
+
 int main (int argc, char **argv)
 {
     char *p;
@@ -2649,7 +2687,7 @@ int main (int argc, char **argv)
     vfs_init ();
     ftpfs_init_passwd ();
 
-    if (recurs_dir || copy_mode) {
+    if (recurs_mode || copy_mode) {
 	if (!expand_glob (&control_files)) {
 	    progmess ("error on cmdline near", "no files match glob expressions");
 	    exit (RETURN_VALUE_ERROR);
@@ -2657,16 +2695,23 @@ int main (int argc, char **argv)
     }
 
     if (secure_login) {
+	int login_failed;
 	char s[MAX_PATH_LEN];
 	if (strncmp (control_files[0], "mc:" PATH_SEP_STR PATH_SEP_STR, 5)) {
 	    sprintf (s, "mc:" PATH_SEP_STR PATH_SEP_STR "%s", control_files[0]);
 	} else {
 	    sprintf (s, "%s", control_files[0]);
 	}
-	mcfs_open_login (s);		/* should not return */
-	progmess ("cannot login to", control_files[0]);
-	mad_finalize (__FILE__, __LINE__);
-	exit (1);
+	login_failed = mcfs_open_login (s, test_login);
+	if (!test_login) {
+	    progmess ("cannot login to", control_files[0]);
+	    mad_finalize (__FILE__, __LINE__);
+	    exit (1);
+	}
+	if (login_failed)
+	    exit (1);
+	else
+	    exit (0);
     } else if (tar_output) {
 	tar_init (tar_file_name);
 	r = recurs_dir (0, "." PATH_SEP_STR, control_files);
