@@ -145,6 +145,7 @@ char *lookup_netrc (char *host, char *user)
 
 struct passwd *cached_getpwuid (pid_t pid);
 
+#if 0	/* huh? can't use the uid for the home directory -- what about $HOME ? */
 static void name_netrc (char *line, int len)
 {
     struct passwd *pw;
@@ -156,6 +157,17 @@ static void name_netrc (char *line, int len)
     strncpy (line, pw->pw_dir, len - 1);
     strncat (line, NETRC, len - 1);
 }
+#else
+extern char *home_dir;
+
+static void name_netrc (char *line, int len)
+{
+    memset (line, 0, len);
+    strncpy (line, home_dir, len - 2 - sizeof (NETRC));
+    strcat (line, PATH_SEP_STR);
+    strcat (line, NETRC);
+}
+#endif
 
 static char *make_token (char *p, char **next)
 {
