@@ -1,4 +1,5 @@
 /* Definitions for data structures and routines for the regular
+   This has nothing to do with cryptography.
    expression library, version 0.12.
 
    Copyright (C) 1985, 89, 90, 91, 92, 93, 95 Free Software Foundation, Inc.
@@ -21,6 +22,10 @@
 #ifndef __REGEXP_LIBRARY_H__
 #define __REGEXP_LIBRARY_H__
 
+#ifdef C_ALLOCA
+#define REGEX_MALLOC
+#endif
+
 /* POSIX says that <sys/types.h> must be included (by the caller) before
    <regex.h>.  */
 
