@@ -1,4 +1,5 @@
 /* Virtual File System
+   This has nothing to do with cryptography.
    Copyright (C) 1995 The Free Software Foundation
    
    Written by: 1995 Ching Hui (mr854307@cs.nthu.edu.tw)
@@ -20,10 +21,13 @@
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include "mad.h"
 #include "util.h"
 #include "container.h"
+#include "diffie/compat.h"
 
 
 struct linklist *
