@@ -62,6 +62,9 @@
 static int ftpfserrno;
 static int code;
 
+/* if on, does not prepend a - to the anonymous ftp password */
+int ftpfs_literal_password = 0;
+
 /* Delay to retry a connection */
 int ftpfs_retry_seconds = 30;
 
@@ -73,6 +76,8 @@ int use_netrc = 1;
 
 char *home_dir = 0;
 
+extern int verbose;
+
 char *ftpfs_temp_name_dir = 0;
 
 /* Anonymous setup */
@@ -200,6 +205,11 @@ int getreply (int sock, char *string_buf, int string_len)
 	    code = 421;
 	    return 4;
 	}
+	if (verbose >= 3) {
+	    char m[128];
+	    sprintf (m, "ftpfs: server: %.80s\n", answer);
+	    print_vfs_message (m);
+	}
 	switch(sscanf(answer, "%d", &code)) {
 	    case 0:
 	        if (string_buf) {
@@ -313,6 +323,11 @@ static int command (struct ftpfs_connection *bucket, int wait_reply,
     got_sigpipe = 0;
     enable_interrupt_key();
     status = send(sock, buf, strlen(buf), 0);
+    if (verbose >= 4) {
+	char m[128];
+	sprintf (m, "ftpfs:   send: %.80s\n", buf);
+	print_vfs_message (m);
+    }
     if (status < 0){
 	code = 421;
 	if (errno == EPIPE){
@@ -428,7 +443,7 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
         }
     }
 
-    if (!anon || ftpfs_debug_server_dialog)
+    if (!anon || ftpfs_debug_server_dialog || ftpfs_literal_password)
 	pass = (char *) strdup (op);
     else
 	pass = copy_strings ("-", op, 0);
@@ -1425,7 +1440,7 @@ static struct ftpfs_dir *
     dcache->remote_path = (char *) strdup (remote_path);
     dcache->count = 1;
 
-    sock = open_data_connection (bucket, "LIST -la", ".", 0);
+    sock = open_data_connection (bucket, "LIST -la", remote_path, 0);
 
     if (sock == -1)
 	goto error_3;
@@ -1436,6 +1451,12 @@ static struct ftpfs_dir *
     errno = 0;
     while ((got_intr = get_line_interruptible (buffer, sizeof (buffer), sock)) != EINTR) {
 	int eof = got_intr == 0;
+	if (verbose >= 4) {
+	    char m[128];
+	    sprintf (m, "ftpfs: server: %.80s\n", buffer);
+	    print_vfs_message (m);
+	}
+	    
 
 	if (ftpfs_debug_server_dialog) {
 	    fputs (buffer, ftpfs_logfile);
@@ -1673,11 +1694,13 @@ int ftpfs_ctl (void *data, int ctlop, int arg)
 			return MCERR_READ;
 		}
 	        if (n == 0) {
+    	    	    shutdown (remotesock, 2);
+    	    	    close(remotelocal_handle);
+                    remotesock = -1;
+                    remotelocal_handle = -1;
     	    	    if (getreply (qsock (remoteent->bucket), NULL, 0) != COMPLETE) {
 	        	ftpfserrno = EIO;
 	    	    }
-    	    	    close(remotelocal_handle);
-    	    	    shutdown (remotesock, 2);
 		    if (localname){
 			free (localname);
 			localname = NULL;
