@@ -339,8 +339,12 @@ static int get_speed_as_int (speed_t speed)
 	case B38400: return 38400;
 	case B57600: return 57600;
 	case B115200: return 115200;
+#ifdef B230400
 	case B230400: return 230400;
+#endif
+#ifdef B460800
 	case B460800: return 460800;
+#endif
     }
     return 9600;
 }
@@ -421,7 +425,7 @@ FILE *my_stdout;
 
 /* this was written using rlogin as a tutorial. here there is no
    such thing as out out-of-band data */
-int mcfs_open_login (char *url)
+int mcfs_open_login (char *url, int test_login)
 {
     int wait_for_prompt;
 #ifdef B19200
@@ -452,6 +456,10 @@ int mcfs_open_login (char *url)
 	return 1;
     if (version == 3)		/* FIXME */
 	version = 2;
+
+    if (test_login)
+	return (xmcfs_login_server (my_socket, host, user, port, 1, 0, &version, MC_LOGIN) <= 0);
+
     my_socket = xmcfs_login_server (my_socket, host, user, port, 1, 0, &version, MC_SHELL);
 
     if (my_socket <= 0)
@@ -523,15 +531,21 @@ int mcfs_open_login (char *url)
 #endif
 
 #ifdef B19200
+#ifdef OCRNL
     tios.c_oflag &= ~(ONLCR | OCRNL);
+#else
+    tios.c_oflag &= ~ONLCR;
+#endif
     tios.c_lflag &= ~(ECHO | ICANON | ISIG);
     tios.c_iflag &= ~(ICRNL);
     tios.c_cc[VTIME] = 1;
     tios.c_cc[VMIN] = 1;
     if (option_user_eight_bits)
 	tios.c_iflag &= ~(ISTRIP);
+#if defined(TABDLY) && defined(TAB3)
     if ((tios.c_oflag & TABDLY) == TAB3)
 	tios.c_oflag &= ~TAB3;
+#endif
 /* disable interpretation of ^S: */
     tios.c_iflag &= ~IXON;
 /* no idea if we need these #ifdef's but they look cool */
