@@ -1,4 +1,5 @@
 /* Look up user and/or group names.
+   This has nothing to do with cryptography.
    Copyright (C) 1988, 1992 Free Software Foundation
 
    From GNU Tar.
@@ -25,17 +26,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  */
 
 #include <config.h>
-#include <sys/types.h>
-#include <string.h>
-#include <unistd.h>
-#define TAR_NAMES
-#include "tar.h"
-#include "names.h"
-
-#include <stdio.h>
-#include <pwd.h>
-#include <grp.h>
-#include "mad.h"
+#include "net-includes.h"
+#include "diffie/compat.h"
 
 #define TUNMLEN 256
 #define TGNMLEN 256
@@ -51,6 +43,9 @@ static int my_gid = -993;
 #define myuid	( my_uid < 0? (my_uid = getuid()): my_uid )
 #define	mygid	( my_gid < 0? (my_gid = getgid()): my_gid )
 
+struct passwd *cached_getpwuid (pid_t pid);
+struct group *cached_getgrgid (gid_t gid);
+
 /*
  * Look up a user or group name from a uid/gid, maintaining a cache.
  * FIXME, for now it's a one-entry cache.
@@ -62,14 +57,10 @@ static int my_gid = -993;
 void finduname (char *uname, int uid)
 {
     struct passwd *pw;
-#ifndef HAVE_GETPWUID
-    extern struct passwd *getpwuid ();
-#endif
-
     if (uid != saveuid) {
 	saveuid = uid;
 	saveuname[0] = '\0';
-	pw = getpwuid (uid);
+	pw = cached_getpwuid (uid);
 	if (pw)
 	    strncpy (saveuname, pw->pw_name, TUNMLEN);
     }
@@ -98,15 +89,11 @@ int finduid (char *uname)
 void findgname (char *gname, int gid)
 {
     struct group *gr;
-#ifndef HAVE_GETGRGID
-    extern struct group *getgrgid ();
-#endif
-
     if (gid != savegid) {
 	savegid = gid;
 	savegname[0] = '\0';
-	(void) setgrent ();
-	gr = getgrgid (gid);
+	setgrent ();
+	gr = cached_getgrgid (gid);
 	if (gr)
 	    strncpy (savegname, gr->gr_name, TGNMLEN);
     }
@@ -131,3 +118,104 @@ int findgid (char *gname)
     }
     return savegid;
 }
+
+struct passwd_list {
+    struct passwd *p;
+    pid_t pid;
+    struct passwd_list *next;
+};
+
+struct passwd *cached_getpwuid (pid_t pid)
+{
+    static struct passwd_list *l = 0, *last = 0;
+    struct passwd_list *p;
+    struct passwd *r;
+    if (last)
+	if (last->pid == pid)
+	    return last->p;
+    for (p = l; p; p = p->next)
+	if (p->pid == pid) {
+	    last = p;
+	    return p->p;
+	}
+    r = getpwuid (pid);
+    p = l;
+    l = (struct passwd_list *) malloc (sizeof (struct passwd_list));
+    if (r) {
+	l->p = (struct passwd *) malloc (sizeof (struct passwd));
+	memcpy (l->p, r, sizeof (struct passwd));
+	r = l->p;
+	if (r->pw_name)
+	    r->pw_name = (char *) strdup (r->pw_name);
+	if (r->pw_passwd)
+	    r->pw_passwd = (char *) strdup (r->pw_passwd);
+	if (r->pw_gecos)
+	    r->pw_gecos = (char *) strdup (r->pw_gecos);
+	if (r->pw_dir)
+	    r->pw_dir = (char *) strdup (r->pw_dir);
+	if (r->pw_shell)
+	    r->pw_shell = (char *) strdup (r->pw_shell);
+    } else {
+	l->p = 0;
+    }
+    l->pid = pid;
+    l->next = p;
+    last = l;
+    return l->p;
+}
+
+struct group_list {
+    struct group *g;
+    gid_t gid;
+    struct group_list *next;
+};
+
+struct group *cached_getgrgid (gid_t gid)
+{
+    static struct group_list *l = 0, *last = 0;
+    struct group_list *g;
+    struct group *r;
+    if (last)
+	if (last->gid == gid)
+	    return last->g;
+    for (g = l; g; g = g->next)
+	if (g->gid == gid) {
+	    last = g;
+	    return g->g;
+	}
+    r = getgrgid (gid);
+    g = l;
+    l = (struct group_list *) malloc (sizeof (struct group_list));
+    if (r) {
+	l->g = (struct group *) malloc (sizeof (struct group));
+	memcpy (l->g, r, sizeof (struct group));
+	r = l->g;
+	if (r->gr_name)
+	    r->gr_name = (char *) strdup (r->gr_name);
+	if (r->gr_passwd)
+	    r->gr_passwd = (char *) strdup (r->gr_passwd);
+	if (r->gr_mem) {
+	    int i;
+	    char **m;
+	    for (i = 0; r->gr_mem[i]; i++);
+	    m = malloc (sizeof (char *) * (i + 1));
+	    m[i] = 0;
+	    for (i = 0; r->gr_mem[i]; i++)
+		m[i] = (char *) strdup (r->gr_mem[i]);
+	    r->gr_mem = m;
+	}
+    } else {
+	l->g = 0;
+    }
+    l->gid = gid;
+    l->next = g;
+    last = l;
+    return l->g;
+}
+
+
+
+
+
+
+
