@@ -6,6 +6,9 @@
    Written by: 
       Miguel de Icaza, 1995, 1997, 
       Andrej Borsenkow 1996.
+
+   Heavily modifed by:
+      Paul Sheer 1998, 1999.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,6 +52,12 @@
 #include "diffie/compat.h"
 #include "mad.h"
 
+#ifdef HAVE_WRAP
+#include <tcpd.h>
+int allow_severity = 1;
+int deny_severity = 1;
+#endif
+
 #if defined(HAVE_SHADOW)
 #   if defined(HAVE_PW_ENCRYPT)
 #   elif defined(HAVE_CRYPT)
@@ -74,6 +83,8 @@ struct in_addr client_addr;
 
 char client_user[64] = "nobody";
 
+char *password_server = 0;
+
 /* Requested version number from client */
 static int clnt_version;
 
@@ -910,7 +921,7 @@ int do_classic_auth (char *username, char *password)
 #if defined(HAVE_SHADOW)
     {
 	struct spwd *spw;
-	if (!(spw = getspnam (username)))
+	if (!(spw = (struct spwd *) getspnam (username)))
 	    this->pw_passwd = "*";
 	else
 	    this->pw_passwd = spw->sp_pwdp;
@@ -1074,7 +1085,20 @@ void xdo_login (int set_user)
 	DO_QUIT_VOID ();
     }
     rpc_get (msock, RPC_LIMITED_STRING, &password, RPC_END);
-    logged_in = do_auth (username, password, set_user);
+    if (password_server) {
+	char s[256];
+	FILE *p;
+	sprintf (s, "%s %s@%s --test-login --read-password-from-stdin --no-warn-first-login", AUTH_PROG, username, password_server);
+	p = popen (s, "w");
+	if (!p) {
+	    printf ("do_login: popen failed\n");
+	} else {
+	    fwrite (password, strlen (password), 1, p);
+	    logged_in = !pclose (p);
+	}
+    } else {
+	logged_in = do_auth (username, password, set_user);
+    }
     endpwent ();
     login_reply (logged_in);
 }
@@ -1580,6 +1604,9 @@ char *get_client (int port_number)
     struct hostent *hp;
     char hostname[255];
     int yes = 1;
+#ifdef HAVE_WRAP
+    struct request_info curr_request;
+#endif
 
     if ((sock = socket (AF_INET, SOCK_STREAM, 0)) < 0)
 	return "Can't create socket";
@@ -1629,6 +1656,14 @@ char *get_client (int port_number)
 	}
 	if (isDaemon && fork ())
 	    exit (0);
+#ifdef HAVE_WRAP
+	request_init (&curr_request, RQ_DAEMON, "secure-mcserv",
+		 RQ_CLIENT_ADDR, inet_ntoa (client_address.sin_addr), 0);
+	if (!hosts_access (&curr_request)) {
+	    close (newsocket);
+	    return 0;
+	}
+#endif
 #if defined(IP_TOS) && 0
 	on = IPTOS_LOWDELAY;
 	setsockopt (newsocket, IPPROTO_IP, IP_TOS, (char *) &on, sizeof (int));
@@ -1735,7 +1770,7 @@ int main (int argc, char *argv [])
     diffie_add_functions ();
     parser_init ();
 
-    while ((c = getopt (argc, argv, "fdiqp:t:v")) != -1){
+    while ((c = getopt (argc, argv, "fdiqs:p:t:v")) != -1){
 	switch (c){
 	case 'd':
 	    isDaemon = 1;
@@ -1770,6 +1805,10 @@ int main (int argc, char *argv [])
 	    r_auth = 1;
 	    break;
 
+	case 's':
+	    password_server = (char *) strdup (optarg);
+	    break;
+
 	default:
 	    fprintf (stderr, 
 		    "secure-mcserv from Mirrordir version " VERSION " "
