@@ -1,6 +1,5 @@
 /* Operations of the rpc manager */
-
-#include <netinet/in.h>	/* For sockaddr_in needed by get_remote_port */
+/*   This has nothing to do with cryptography. */
 
 /* Please note that the RPC manager does not use integers, it only uses */
 /* 4-byte integers for the comunication */
@@ -19,7 +18,7 @@ int socket_read_block (int sock, char *dest, int len);
 int socket_write_block (int sock, char *buffer, int len);
 int send_string (int sock, char *string);
 void tcp_init (void);
-int get_remote_port (struct sockaddr_in *sin, int *version);
+int get_remote_port (struct sockaddr_in *si, int *version);
 int open_tcp_link  (char *host, int *port, int *version, char *caller);
 
 #ifdef HAVE_MAD
