@@ -1766,8 +1766,9 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
     num_cols = split_text (p);
 
     s->st_nlink = atol (columns [0]);
-    if (s->st_nlink <= 0)
-        free_and_return (0);
+    /* some servers return "0" as st_nlink (must be handled) */
+    if (s->st_nlink < 0 || (s->st_nlink == 0 && strcmp (columns[0],"0")))
+	free_and_return (0);
 
     if (!is_num (1))
 	s->st_uid = finduid (columns [1]);
