@@ -1,13 +1,7 @@
+/*   This has nothing to do with cryptography. */
 #ifndef __VFS_H
 #define __VFS_H
 
-/* #include <src/mostincludes.h> */
-
-#if !defined(SCO_FLAVOR) || !defined(_SYS_SELECT_H)
-#	include <sys/time.h>	/* alex: this redefines struct timeval */
-#endif /* SCO_FLAVOR */
-#include <utime.h>
-
 #include "vfs/util.h"
 
 #ifdef USE_VFS
@@ -36,7 +30,7 @@
 	int  (*lstat)(char *path, struct stat *buf);
 	int  (*fstat)(void *vfs_info, struct stat *buf);
 
-	int  (*chmod)(char *path, int mode);
+	int  (*chmod)(char *path, unsigned long mode);
 	int  (*chown)(char *path, int owner, int group);
 	int  (*utime)(char *path, struct utimbuf *times);
 
@@ -48,7 +42,7 @@
 	int  (*chdir)(char *path);
 	int  (*ferrno)(void);
 	int  (*lseek)(void *vfs_info, off_t offset, int whence);
-	int  (*mknod)(char *path, int mode, int dev);
+	int  (*mknod)(char *path, int mode, dev_t dev);
 	
 	vfsid (*getid)(char *path, struct vfs_stamping **parent);
 	int  (*nothingisopen)(vfsid id);
@@ -57,7 +51,7 @@
 	char *(*getlocalcopy)(char *filename);
 	void (*ungetlocalcopy)(char *filename, char *local, int has_changed);
 
-	int  (*mkdir)(char *path, mode_t mode);
+	int  (*mkdir)(char *path, unsigned long mode);
 	int  (*rmdir)(char *path);
 	
 	int  (*ctl)(void *vfs_info, int ctlop, int arg);
@@ -123,8 +117,8 @@
     void vfs_add_noncurrent_stamps (vfs *, vfsid, struct vfs_stamping *);
     void vfs_add_current_stamps (void);
     void vfs_free_resources(char *path);
-    void vfs_timeout_handler ();
-    int vfs_timeouts ();
+    void vfs_timeout_handler (void);
+    int vfs_timeouts (void);
     void vfs_force_expire (char *pathname);
 
     void vfs_fill_names (void (*)(char *));
@@ -177,11 +171,11 @@
 	int mc_unlink (char *path);
 	int mc_symlink (char *name1, char *name2);
         int mc_link (char *name1, char *name2);
-        int mc_mknod (char *, int, int);
+        int mc_mknod (char *, int, dev_t);
 	int mc_rename (char *original, char *target);
 	int mc_write (int fd, char *buf, int nbyte);
         int mc_rmdir (char *path);
-        int mc_mkdir (char *path, mode_t mode);
+        int mc_mkdir (char *path, unsigned long mode);
         char *mc_getlocalcopy (char *filename);
         void mc_ungetlocalcopy (char *filename, char *local, int has_changed);
         char *mc_def_getlocalcopy (char *filename);
@@ -285,6 +279,8 @@
 int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname);
 #endif
 
+void char_translate (char *p, char a, char b);
+
 #define MCCTL_SETREMOTECOPY	0
 #define MCCTL_ISREMOTECOPY	1
 #define MCCTL_REMOTECOPYCHUNK	2
@@ -293,6 +289,7 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname);
 #define MCCTL_REMOVELOCALCOPY   5
 #define MCCTL_SETDIRTIMEOUT	6
 #define MCCTL_SETTMPDIR		7
+#define MCCTL_QUICKFILEREAD     8
 
 /* Return codes from the ${fs}_ctl routine */
 
